package dev.boyfailure.quajra.vectorbreakout;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import javax.swing.JComponent;
import java.awt.geom.Line2D;

public class GameDisplay extends JComponent {
    
    // Change this to 1 if you build for older versions of Java. Older versions
    // will render a leading zero otherwise. (tested on JDK 1.5)
    byte iStart = 0;
    
    float beamX = 0; // Horizontal location of "vector beam"
    float beamY = 0; // Vertical location of "vector beam"
    
    public static float beamScaleX = 1; // Horizontal drawing scale
    public static float beamScaleY = 1; // Vertical drawing scale
    public static float beamThicknessScale = 1; // Thickness of lines
    
    private GameState gameState;
    
    public GameDisplay(GameState gameState) {
        this.gameState = gameState;
    }
    
    public void paint(Graphics gameGraphics) {
        
        super.paint(gameGraphics);
        
        Graphics2D g2 = (Graphics2D) gameGraphics;
        
        // Walls
        if (gameState.isGameStarted) {
            g2.setColor(Color.gray);
            g2.setStroke(bt(10));
            resetBeam();
            moveBeam(15, 0);
            g2.draw(drawVec(0, 600));
            moveBeam(770, 0);
            g2.draw(drawVec(0, -600));
            moveBeam(15, -5);
            g2.draw(drawVec(-800, 0));
        }
        
        // Bricks
        if (gameState.currentLevel != null) {
            for (short i = 0; i < gameState.currentLevel.levelBricks.length; i++) {
                if (gameState.currentLevel.levelBricks[i] != null) {
                    if (gameState.currentLevel.levelBricks[i].isBroken == false) {
                        resetBeam();
                        g2.setStroke(bt(3));
                        moveBeam(gameState.currentLevel.levelBricks[i].brickX + 1, gameState.currentLevel.levelBricks[i].brickY + 1);
                        g2.setColor(vc(gameState.currentLevel.levelBricks[i].brickColor));
                        g2.draw(drawVec(gameState.currentLevel.levelBricks[i].brickWidth - 2, 0));
                        g2.draw(drawVec(0, -(gameState.currentLevel.levelBricks[i].brickHeight - 2)));
                        g2.draw(drawVec(-(gameState.currentLevel.levelBricks[i].brickWidth - 2), 0));
                        g2.draw(drawVec(0, gameState.currentLevel.levelBricks[i].brickHeight - 2));
                    }
                }
            }
        }
        
        // Paddles
        g2.setColor(Color.white);
        for (byte i = 0; i < gameState.currentPaddles.length; i++) {
            if (gameState.currentPaddles[i] != null) {
                if (gameState.currentPaddles[i].isActive) {
                    resetBeam();
                    g2.setStroke(bt(3));
                    moveBeam(gameState.currentPaddles[i].paddleX, gameState.currentPaddles[i].paddleY);
                    g2.draw(drawVec(gameState.currentPaddles[i].paddleWidth, 0));
                    g2.draw(drawVec(0, -(gameState.currentPaddles[i].paddleHeight)));
                    g2.draw(drawVec(-(gameState.currentPaddles[i].paddleWidth), 0));
                    g2.draw(drawVec(0, gameState.currentPaddles[i].paddleHeight));
                }
            }
        }
        
        // Balls
        g2.setStroke(bt(3));
        g2.setColor(Color.yellow);
        for (byte i = 0; i < gameState.currentBalls.length; i++) {
            if (gameState.currentBalls[i] != null) {
                if (gameState.currentBalls[i].isActive) {
                    resetBeam();
                    moveBeam(gameState.currentBalls[i].ballX, gameState.currentBalls[i].ballY);
                    g2.draw(drawVec(gameState.currentBalls[i].ballWidth, 0));
                    g2.draw(drawVec(0, -gameState.currentBalls[i].ballHeight));
                    g2.draw(drawVec(-gameState.currentBalls[i].ballWidth, 0));
                    g2.draw(drawVec(0, gameState.currentBalls[i].ballHeight));
                }
            }
        }
        
        // Particles
        for (byte i = 0; i < gameState.currentParticles.length; i++) {
            if (gameState.currentParticles[i] != null) {
                if (gameState.currentParticles[i].isActive) {
                    resetBeam();
                    moveBeam(gameState.currentParticles[i].particleX, gameState.currentParticles[i].particleY);
                    g2.setStroke(bt(gameState.currentParticles[i].vectorScale));
                    g2.setColor(vc(gameState.currentParticles[i].particleColor));
                    g2.draw(drawVec(gameState.currentParticles[i].particleX2, gameState.currentParticles[i].particleY2));
                }
            }
        }
        
        /* TextElements
         *
         * Note on rendering numbers:
         * 
         * I am aware that I could be using a switch statement for these.
         * However, this did not compile when testing in JDK 1.5, as it would
         * refuse to accept anything other than numbers in a switch statement.
         * Since I am a silly little nerd who wishes to use this on my older
         * systems, I used these goofy if/else statements so I could compile
         * on the older JDKs and play on my older systems...
         */
        for (byte i = 0; i < gameState.currentTextElements.length; i++) {
            if (gameState.currentTextElements[i] != null) {
                if (gameState.currentTextElements[i].active) {
                    resetBeam();
                    moveBeam(gameState.currentTextElements[i].x, gameState.currentTextElements[i].y);

                    // Save the current beam scale to revert once TextElement is done rendering
                    float oldBeamScaleX = beamScaleX;
                    float oldBeamScaleY = beamScaleY;
                    
                    // Adjust the beam scale to accomodate the scale of the TextElement
                    beamScaleX = beamScaleX * gameState.currentTextElements[i].textScale;
                    beamScaleY = beamScaleY * gameState.currentTextElements[i].textScale;

                    // Render the TextElement
                    g2.setColor(vc(gameState.currentTextElements[i].textColor));
                    g2.setStroke(bt(gameState.currentTextElements[i].vectorScale));
                    String stringToSplit = String.valueOf(gameState.currentTextElements[i].text);
                    String[] stringToRender = stringToSplit.split("");
                    //<editor-fold defaultstate="collapsed" desc=" Beam routines for letters/numbers ">
                    for (short ii = iStart; ii < stringToRender.length; ii++) {
                        if ("1".equals(stringToRender[ii])) {
                            moveBeam(7, 0);
                            g2.draw(drawVec(0, -24));
                            moveBeam(11, 24);
                        }
                        else if ("2".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, 12));
                            g2.draw(drawVec(-14, 0));
                            g2.draw(drawVec(0, 12));
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 0);
                        }
                        else if ("3".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, 12));
                            g2.draw(drawVec(-14, 0));
                            moveBeam(14, 0);
                            g2.draw(drawVec(0, 12));
                            g2.draw(drawVec(-14, 0));
                            moveBeam(18, 0);
                        }
                        else if ("4".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(0, 12));
                            g2.draw(drawVec(14, 0));
                            moveBeam(0, -12);
                            g2.draw(drawVec(0, 24));
                            moveBeam(4, 0);
                        }
                        else if ("5".equals(stringToRender[ii]) || "S".equals(stringToRender[ii])) {
                            moveBeam(14, -24);
                            g2.draw(drawVec(-14, 0));
                            g2.draw(drawVec(0, 12));
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, 12));
                            g2.draw(drawVec(-14, 0));
                            moveBeam(18, 0);
                        }
                        else if ("6".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(0, 24));
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, -12));
                            g2.draw(drawVec(-14, 0));
                            moveBeam(18, 12);
                        }
                        else if ("7".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, 24));
                            moveBeam(4, 0);
                        }
                        else if ("8".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, 24));
                            g2.draw(drawVec(-14, 0));
                            moveBeam(0, -12);
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 12);
                        }
                        else if ("9".equals(stringToRender[ii])) {
                            moveBeam(14, 0);
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(-14, 0));
                            g2.draw(drawVec(0, 12));
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 12);
                        }
                        else if ("0".equals(stringToRender[ii]) || "O".equals(stringToRender[ii])) {
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(-14, 0));
                            g2.draw(drawVec(0, 24));
                            moveBeam(18, 0);
                        }
                        // Simulates a newline in a TextElement
                        else if ("\\".equals(stringToRender[ii])) {
                            moveBeam(-(beamX - gameState.currentTextElements[i].x), 30);
                        }
                        else if ("A".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -17));
                            g2.draw(drawVec(7, -7));
                            g2.draw(drawVec(7, 7));
                            g2.draw(drawVec(0, 17));
                            moveBeam(-14, -10);
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 10);
                        }
                        else if ("B".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(11, 0));
                            g2.draw(drawVec(3, 3));
                            g2.draw(drawVec(0, 6));
                            g2.draw(drawVec(-3, 3));
                            g2.draw(drawVec(3, 3));
                            g2.draw(drawVec(0, 6));
                            g2.draw(drawVec(-3, 3));
                            g2.draw(drawVec(-11, 0));
                            moveBeam(0, -12);
                            g2.draw(drawVec(11, 0));
                            moveBeam(7, 12);
                        }
                        else if ("C".equals(stringToRender[ii])) {
                            moveBeam(14, -24);
                            g2.draw(drawVec(-14, 0));
                            g2.draw(drawVec(0, 24));
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 0);
                        }
                        else if ("D".equals(stringToRender[ii])) {
                            g2.draw(drawVec(7, 0));
                            g2.draw(drawVec(7, -7));
                            g2.draw(drawVec(0, -10));
                            g2.draw(drawVec(-7, -7));
                            g2.draw(drawVec(-7, 0));
                            g2.draw(drawVec(0, 24));
                            moveBeam(18, 0);
                        }
                        else if ("E".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(14, 0));
                            moveBeam(-14, 12);
                            g2.draw(drawVec(14, 0));
                            moveBeam(-14, 12);
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 0);
                        }
                        else if ("F".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(14, 0));
                            moveBeam(-14, 12);
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 12);
                        }
                        else if ("G".equals(stringToRender[ii])) {
                            moveBeam(14, -17);
                            g2.draw(drawVec(0, -7));
                            g2.draw(drawVec(-14, 0));
                            g2.draw(drawVec(0, 24));
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, -8));
                            g2.draw(drawVec(-8, 0));
                            moveBeam(12, 8);
                        }
                        else if ("H".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            moveBeam(14, 0);
                            g2.draw(drawVec(0, 24));
                            moveBeam(-14, -12);
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 12);
                        }
                        else if ("I".equals(stringToRender[ii])) {
                            g2.draw(drawVec(14, 0));
                            moveBeam(-14, -24);
                            g2.draw(drawVec(14, 0));
                            moveBeam(-7, 0);
                            g2.draw(drawVec(0, 24));
                            moveBeam(11, 0);
                        }
                        else if ("J".equals(stringToRender[ii])) {
                            moveBeam(0, -7);
                            g2.draw(drawVec(7, 7));
                            g2.draw(drawVec(7, 0));
                            g2.draw(drawVec(0, -24));
                            moveBeam(4, 24);
                        }
                        else if ("K".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            moveBeam(14, 0);
                            g2.draw(drawVec(-14, 12));
                            g2.draw(drawVec(14, 12));
                            moveBeam(4, 0);
                        }
                        else if ("L".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(0, 24));
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 0);
                        }
                        else if ("M".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(7, 7));
                            g2.draw(drawVec(7, -7));
                            g2.draw(drawVec(0, 24));
                            moveBeam(4, 0);
                        }
                        else if ("N".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(14, 24));
                            g2.draw(drawVec(0, -24));
                            moveBeam(4, 24);
                        }
                        else if ("P".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, 12));
                            g2.draw(drawVec(-14, 0));
                            moveBeam(18, 12);
                        }
                        else if ("Q".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, 18));
                            g2.draw(drawVec(-6, 6));
                            g2.draw(drawVec(-8, 0));
                            moveBeam(8, -6);
                            g2.draw(drawVec(6, 6));
                            moveBeam(4, 0);
                        }
                        else if ("R".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, 12));
                            g2.draw(drawVec(-14, 0));
                            moveBeam(2, 0);
                            g2.draw(drawVec(12, 12));
                            moveBeam(4, 0);
                        }
                        else if ("T".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(14, 0));
                            moveBeam(-7, 0);
                            g2.draw(drawVec(0, 24));
                            moveBeam(11, 0);
                        }
                        else if ("U".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(0, 24));
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, -24));
                            moveBeam(4, 24);
                        }
                        else if ("V".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(7, 24));
                            g2.draw(drawVec(7, -24));
                            moveBeam(4, 24);
                        }
                        else if ("W".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(0, 24));
                            g2.draw(drawVec(7, -7));
                            g2.draw(drawVec(7, 7));
                            g2.draw(drawVec(0, -24));
                            moveBeam(4, 24);
                        }
                        else if ("X".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(14, 24));
                            moveBeam(-14, 0);
                            g2.draw(drawVec(14, -24));
                            moveBeam(4, 24);
                        }
                        else if ("Y".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(7, 7));
                            g2.draw(drawVec(7, -7));
                            moveBeam(-7, 7);
                            g2.draw(drawVec(0, 17));
                            moveBeam(11, 0);
                        }
                        else if ("Z".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(-14, 24));
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 0);
                        }
                        else if (".".equals(stringToRender[ii])) {
                            moveBeam(7, 0);
                            g2.draw(drawVec(0, -2));
                            moveBeam(11, 2);
                        }
                        else if (",".equals(stringToRender[ii])) {
                            moveBeam(7, 0);
                            g2.draw(drawVec(0, -5));
                            moveBeam(11, 5);
                        }
                        else if (":".equals(stringToRender[ii])) {
                            moveBeam(7, 0);
                            g2.draw(drawVec(0, -2));
                            moveBeam(0, -10);
                            g2.draw(drawVec(0, -2));
                            moveBeam(11, 14);
                        }
                        else if (";".equals(stringToRender[ii])) {
                            moveBeam(7, 0);
                            g2.draw(drawVec(0, -5));
                            moveBeam(0, -7);
                            g2.draw(drawVec(0, -2));
                            moveBeam(11, 14);
                        }
                        else if ("!".equals(stringToRender[ii])) {
                            moveBeam(7, 0);
                            g2.draw(drawVec(0, -2));
                            moveBeam(0, -10);
                            g2.draw(drawVec(0, -12));
                            moveBeam(11, 24);
                        }
                        else if ("(".equals(stringToRender[ii])) {
                            moveBeam(10, 0);
                            g2.draw(drawVec(-6, -6));
                            g2.draw(drawVec(0, -12));
                            g2.draw(drawVec(6, -6));
                            moveBeam(8, 24);
                        }
                        else if (")".equals(stringToRender[ii])) {
                            moveBeam(10, 0);
                            g2.draw(drawVec(6, -6));
                            g2.draw(drawVec(0, -12));
                            g2.draw(drawVec(-6, -6));
                            moveBeam(8, 24);
                        }
                        else { // Blank space
                            moveBeam(18, 0);
                        }
                    }
                    //</editor-fold>

                    // Revert back to the previous beam scale
                    beamScaleX = oldBeamScaleX;
                    beamScaleY = oldBeamScaleY;
                }
            }
        }
        
        gameState.actualFrames++; // Count up actual frames rendered
        
        // Debug Menu
        if (gameState.debugMenuEnabled) {
            g2.setColor(Color.white);
            g2.drawString(gameState.gameName + " v" + gameState.gameVersion, 1, 12);
            g2.drawString("frame: " + gameState.frameCounter, 1, 24);
            g2.drawString("beamScaleX: " + beamScaleX, 1, 36);
            g2.drawString("beamScaleY: " + beamScaleY, 1, 48);
            g2.drawString("isPaused: " + gameState.isPaused, 1, 60);
            g2.drawString("targetFrameRate: " + gameState.targetFrameRate, 150, 24);
            g2.drawString("actualFrameRate: " + gameState.actualFrameRate, 150, 36);
            g2.drawString("actualFrames: " + gameState.actualFrames, 150, 48);
            g2.drawString("chancey: " + gameState.chancey, 150, 60);
            g2.drawString("ballsOnScreen: " + gameState.ballsOnScreen, 300, 24);
            short padBoxX, padBoxY, padBoxX2, padBoxY2;
            g2.setStroke(new BasicStroke(1));
            // Paddles
            g2.setColor(vc(10, 0.5f));
            for (byte i = 0; i < gameState.currentPaddles.length; i++) {
                if (gameState.currentPaddles[i] != null) {
                    padBoxX = (short) gameState.currentPaddles[i].hitBox.bounds.getBounds2D().getMinX();
                    padBoxY = (short) gameState.currentPaddles[i].hitBox.bounds.getBounds2D().getMinY();
                    padBoxX2 = (short) gameState.currentPaddles[i].hitBox.bounds.getBounds2D().getWidth();
                    padBoxY2 = (short) gameState.currentPaddles[i].hitBox.bounds.getBounds2D().getHeight();
                    g2.fillRect(padBoxX, padBoxY + (40 - gameState.currentPaddles[i].paddleHeight), padBoxX2, padBoxY2);
                    padBoxX = (short) gameState.currentPaddles[i].hitBoxLeft.bounds.getBounds2D().getMinX();
                    padBoxY = (short) gameState.currentPaddles[i].hitBoxLeft.bounds.getBounds2D().getMinY();
                    padBoxX2 = (short) gameState.currentPaddles[i].hitBoxLeft.bounds.getBounds2D().getWidth();
                    padBoxY2 = (short) gameState.currentPaddles[i].hitBoxLeft.bounds.getBounds2D().getHeight();
                    g2.fillRect(padBoxX, padBoxY + (40 - gameState.currentPaddles[i].paddleHeight), padBoxX2, padBoxY2);
                    padBoxX = (short) gameState.currentPaddles[i].hitBoxRight.bounds.getBounds2D().getMinX();
                    padBoxY = (short) gameState.currentPaddles[i].hitBoxRight.bounds.getBounds2D().getMinY();
                    padBoxX2 = (short) gameState.currentPaddles[i].hitBoxRight.bounds.getBounds2D().getWidth();
                    padBoxY2 = (short) gameState.currentPaddles[i].hitBoxRight.bounds.getBounds2D().getHeight();
                    g2.fillRect(padBoxX, padBoxY + (40 - gameState.currentPaddles[i].paddleHeight), padBoxX2, padBoxY2);
                }
            }
            // Balls
            g2.setColor(vc(11, 1.0f));
            for (byte i = 0; i < gameState.currentBalls.length; i++) {
                if (gameState.currentBalls[i] != null) {
                    padBoxX = (short) gameState.currentBalls[i].hitBox.bounds.getBounds2D().getMinX();
                    padBoxY = (short) gameState.currentBalls[i].hitBox.bounds.getBounds2D().getMinY();
                    padBoxX2 = (short) gameState.currentBalls[i].hitBox.bounds.getBounds2D().getWidth();
                    padBoxY2 = (short) gameState.currentBalls[i].hitBox.bounds.getBounds2D().getHeight();
                    g2.fillRect(padBoxX, padBoxY + (40 - gameState.currentBalls[i].ballHeight), padBoxX2, padBoxY2);
                }
            }
            // Bricks
            g2.setColor(vc(12, 0.5f));
            for (short i = 0; i < gameState.currentLevel.levelBricks.length; i++) {
                if (gameState.currentLevel.levelBricks[i] != null) {
                    padBoxX = (short) gameState.currentLevel.levelBricks[i].hitBox.bounds.getBounds2D().getMinX();
                    padBoxY = (short) gameState.currentLevel.levelBricks[i].hitBox.bounds.getBounds2D().getMinY();
                    padBoxX2 = (short) gameState.currentLevel.levelBricks[i].hitBox.bounds.getBounds2D().getWidth();
                    padBoxY2 = (short) gameState.currentLevel.levelBricks[i].hitBox.bounds.getBounds2D().getHeight();
                    g2.fillRect(padBoxX, padBoxY + (40 - gameState.currentLevel.levelBricks[i].brickHeight), padBoxX2, padBoxY2);
                }
            }
        }
        
    }
    
    public Line2D.Float drawVec(float x, float y) {
        float oldBeamX = beamX;
        float oldBeamY = beamY;
        beamX += (x * beamScaleX);
        beamY += (y * beamScaleY);
        return new Line2D.Float(oldBeamX, oldBeamY, beamX, beamY);
    }
    
    public Color vc(int colorID) {
        return vc(colorID, 1.0f);
    }
    
    public Color vc(int colorID, float alpha) {
        switch ((byte) colorID) {
            case 0: return new Color(0, 0, 0, alpha); // black
            // default is white but will be written elsewhere as 1
            case 2: return new Color(0.66f, 0.66f, 0.66f, alpha); // grey
            case 3: return new Color(0.5f, 0.5f, 0.5f, alpha); // dark grey
            case 4: return new Color(0.66f, 0, 0, alpha); // dark red
            case 5: return new Color(0, 0.66f, 0, alpha); // dark green
            case 6: return new Color(0, 0, 0.66f, alpha); // dark blue
            case 7: return new Color(1, 0, 0, alpha); // red
            case 8: return new Color(0, 1, 0, alpha); // green
            case 9: return new Color(0, 0, 1, alpha); // blue
            case 10: return new Color(1, 0.5f, 0.5f, alpha); // light red
            case 11: return new Color(0.5f, 1, 0.5f, alpha); // light green
            case 12: return new Color(0.5f, 0.5f, 1, alpha); // light blue
            case 13: return new Color(1, 0.66f, 0, alpha); // orange
            case 14: return new Color(1, 1, 0, alpha); // yellow
            case 15: return new Color(0, 0.66f, 0.66f, alpha); // cyan
            case 16: return new Color(0, 1, 1, alpha); // aqua
            case 17: return new Color(0.66f, 0, 0.66f, alpha); // purple
            case 18: return new Color(1, 0, 1, alpha); // magenta
            case 19: return new Color(1, 0.5f, 0.75f, alpha); // pastel pink
            case 20: return new Color(0.5f, 0.66f, 1, alpha); // pastel blue
            default: return new Color(1, 1, 1, alpha); // white
        }
    }
    
    public BasicStroke bt(float size) {
        return new BasicStroke(size * beamThicknessScale);
    }
    
    public void moveBeam(float x, float y) {
        beamX += (x * beamScaleX);
        beamY += (y * beamScaleY);
    }
    
    public void resetBeam() {
        beamX = 0;
        beamY = 40 * beamScaleY;
    }
    
}
