package dev.boyfailure.quajra.vectorbreakout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

public class GameState {
    
    public String gameName = "Vector Breakout";
    public String gameVersion = "20250511";
    public boolean isGameStarted = false;
    public boolean isPaused = false;
    public boolean debugMenuEnabled = false;
    public boolean isLost = false;
    public boolean movingLeft = false;
    public boolean movingRight = false;
    public boolean confettiMode = false;
    public byte lives = 5;
    public byte chancey = 0;
    public byte ballsOnScreen = 0;
    public byte targetFrameRate = 60;
    public byte actualFrames = 0;
    public byte actualFrameRate = 0;
    public byte gameTickRate = 60;
    public short gameResX = 800;
    public short gameResY = 600;
    public short level = 1;
    public short bricksOnScreen = 0;
    public int score = 0;
    public long frameCounter = 0;
    public Ball[] currentBalls = new Ball[10];
    public Particle[] currentParticles = new Particle[127];
    public Paddle[] currentPaddles = new Paddle[2];
    public TextElement[] currentTextElements = new TextElement[63];
    public Level currentLevel;
    
    // Debug stuffs
    public boolean debugSpawnAllBalls = false;
    public boolean debugUseGiantPaddle = false;
    public short debugStartLevel = 1;
    
    public GameState() {
        this.showTitleScreen();
    }
    
    public KeyListener getKeyListener() {
        return new GameKeyListener();
    }

    public byte getGameTickRate() {
        return gameTickRate;
    }

    public byte getTargetFrameRate() {
        return targetFrameRate;
    }
    
    public ActionListener getGameStateUpdateActionListener() {
        return new GameStateUpdateListener();
    }
    
    public ActionListener getGameTickActionListener() {
        return new GameTickActionListener();
    }
    class GameTickActionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            onGameTick();
        }
    }
    
    class GameKeyListener implements KeyListener {
        public void keyPressed(KeyEvent e) {
            switch(e.getKeyCode()) {
                case 27: // Escape
                    debugMenuEnabled = !debugMenuEnabled;
                    break;
                case 10: // Enter
                    if (isGameStarted) {
                        isPaused = !isPaused;
                        if (!isPaused) {
                            for (byte i = 0; i < currentTextElements.length; i++) {
                                if (currentTextElements[i] != null && "PAUSED".equals(currentTextElements[i].text)) {
                                    currentTextElements[i].cull();
                                    return;
                                }
                            }
                        }
                        else {
                            for (byte i = 0; i < currentTextElements.length; i++) {
                                if (currentTextElements[i] == null) {
                                    currentTextElements[i] = new TextElement(GameState.this, "PAUSED", 5, 142, 310, 1, 4);
                                    currentTextElements[i].activate();
                                    return;
                                }
                            }
                        }
                    }
                    break;
                case 35: // End
                    if (isLost || !isGameStarted) {isGameStarted = true;newGame(true);return;}
                    break;
                case 32: // Space
                    if (isLost || !isGameStarted) {isGameStarted = true;newGame();return;}
                    if (!isPaused) {
                        if (ballsOnScreen < 1) {
                            for (byte i = 0; i < currentBalls.length; i++) {
                                if (currentBalls[i] == null) {
                                    currentBalls[i] = new Ball(GameState.this, 200, 300, 7, 7, 0, true);
                                    currentBalls[i].spawnBall(true);
                                    if (!debugSpawnAllBalls) {return;}
                                }
                            }
                        }
                    }
                    break;
                case 37: // Left
                    if (isGameStarted) {movingLeft = true;}
                    break;
                case 39: // Right
                    if (isGameStarted) {movingRight = true;}
                    break;
            }
        }
        public void keyReleased(KeyEvent e) {
            switch(e.getKeyCode()) {
                case 37: // Left
                    movingLeft = false;
                    break;
                case 39: // Right
                    movingRight = false;
                    break;
            }
        }
        public void keyTyped(KeyEvent e) {}
    }
    
    class GameStateUpdateListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
                bricksOnScreen = 0;
                actualFrameRate = actualFrames;
                actualFrames = 0;
                if (currentLevel != null) {
                    for (short i = 0; i < currentLevel.levelBricks.length; i++) {
                        if (currentLevel.levelBricks[i] != null) {
                            if (currentLevel.levelBricks[i].isBroken) {
                                currentLevel.levelBricks[i] = null;
                            }
                            else {bricksOnScreen++;}
                        }
                    }
                }
                
                for (byte i = 0; i < currentBalls.length; i++) {
                    if (currentBalls[i] != null) {
                        if (currentBalls[i].isActive == false) {
                            currentBalls[i] = null;
                        }
                    }
                }
                for (byte i = 0; i < currentParticles.length; i++) {
                    if (currentParticles[i] != null) {
                        if (currentParticles[i].isActive == false) {
                            currentParticles[i] = null;
                        }
                    }
                }
                for (byte i = 0; i < currentTextElements.length; i++) {
                    if (currentTextElements[i] != null) {
                        if (currentTextElements[i].active == false) {
                            currentTextElements[i] = null;
                        }
                    }
                }

                if (bricksOnScreen <= 0 && isGameStarted && !isLost) {
                    level++;
                    currentLevel = new Level(GameState.this, level);
                    currentLevel.startLevel();
                }
            }
    }
    
    public void incrementFrameCounter() {
        this.frameCounter++;
    }
    
    public void onGameTick() {
        // i will implement something better than "chancey" eventually
        chancey++;
        if (chancey >= 16) {
            chancey = 0;
        }
        
        if (isPaused) {return;}
        
        // Move player if keys are held
        for (byte i = 0; i < currentPaddles.length; i++) {
            if (currentPaddles[i] != null) {
                if (movingLeft) {
                    currentPaddles[i].movePaddle(false);
                }
                else if (movingRight) {
                    currentPaddles[i].movePaddle(true);
                }
                else {
                    currentPaddles[i].paddleSpeed = 0;
                }
            }
        }
        
        // Ball logic (movement, collision checks)
        for (byte i = 0; i < currentBalls.length; i++) {
            if (currentBalls[i] != null) {
                if (currentBalls[i].isActive) {
                    currentBalls[i].moveBall();
                }
            }
        }
        
        // Particles
        for (byte i = 0; i < currentParticles.length; i++) {
            if (currentParticles[i] != null) {
                if (currentParticles[i].isActive) {
                    currentParticles[i].update();
                }
            }
        }
    }
    
    public void showTitleScreen() {
        currentTextElements[0] = new TextElement(this, "VECTOR BREAKOUT", 2.5f, 67, 280, 15, 2);
        currentTextElements[1] = new TextElement(this, "MMXXV BOYFAILURE.DEV", 0.75f, 259, 315, 15, 1);
        currentTextElements[2] = new TextElement(this, "PRESS SPACE TO BEGIN", 1, 215, 345, 15, 2);
        currentTextElements[0].activate();
        currentTextElements[1].activate();
        currentTextElements[2].activate();
    }
    
    public void gameLose() {
        if (currentLevel != null) {
            for (short i = 0; i < currentLevel.levelBricks.length; i++) {
                currentLevel.levelBricks[i] = null;
            }
        }
        for (byte i = 0; i < currentBalls.length; i++) {
            currentBalls[i] = null;
        }
        isLost = true;
        ballsOnScreen = 0;
    }
    
    public void newGame() {
        newGame(false);
    }
    
    public void newGame(boolean debugLevel) {
        for (byte i = 0; i < currentTextElements.length; i++) {
            currentTextElements[i] = null;
        }
        lives = 5;
        isLost = false;
        score = 0;
        if (debugLevel) {level = 32767;}
        else {level = 1;}
        frameCounter = 0;
        ballsOnScreen = 0;
        movingLeft = false;
        movingRight = false;
        if (debugUseGiantPaddle) {currentPaddles[0] = new Paddle(this, 0, 500, 800, 60, 0);}
        else {currentPaddles[0] = new Paddle(this, 350, 500, 100, 15, 0);}
        currentTextElements[0] = new TextElement(this, String.valueOf(score), 1, 20, -13, 1, 2);
        currentTextElements[1] = new TextElement(this, String.valueOf(lives), 1, 200, -13, 1, 2);
        currentTextElements[2] = new TextElement(this, String.valueOf(level), 1, 380, -13, 1, 2);
        currentTextElements[0].activate();
        currentTextElements[1].activate();
        currentTextElements[2].activate();
        if (debugStartLevel > 1) {level = debugStartLevel;}
        currentLevel = new Level(this, level);
        currentLevel.startLevel();
    }
    
}
