package dev.boyfailure.quajra.vectorbreakout;

import java.util.Timer;
import java.util.TimerTask;

public class TextElement {
    
    String text;
    boolean active = false;
    float textScale = 1;
    float vectorScale = 1;
    byte textColor;
    short x;
    short y;
    short duration = 0;
    long creationTime;
    
    private GameState gameState;
    
    /**
     * Text elements.
     * @param gameState The GameState in which the TextElement will reside.
     * @param text The text that will be displayed.
     * @param textScale The size of the text on screen.
     * @param x The x coordinate of the text's origin point.
     * @param y The y coordinate of the text's origin point.
     * @param textColor The color of the text.
     */
    public TextElement(GameState gameState, String text, float textScale, int x, int y, int textColor, float vectorScale) {
        this.gameState = gameState;
        this.text = text;
        this.textScale = textScale;
        this.x = (short) x;
        this.y = (short) y;
        this.textColor = (byte) textColor;
        this.vectorScale = vectorScale;
    }
    
    /**
     * Text elements with a fadeout timer.
     * @param gameState The GameState in which the TextElement will reside.
     * @param text The text that will be displayed.
     * @param textScale The size of the text on screen.
     * @param x The x coordinate of the text's origin point.
     * @param y The y coordinate of the text's origin point.
     * @param textColor The color of the text.
     * @param duration How long the text will stay on screen, in milliseconds.
     */
    public TextElement(GameState gameState, String text, float textScale, int x, int y, int textColor, float vectorScale, int duration) {
        this.gameState = gameState;
        this.text = text;
        this.textScale = textScale;
        this.x = (short) x;
        this.y = (short) y;
        this.textColor = (byte) textColor;
        this.vectorScale = vectorScale;
        this.duration = (short) duration;
        this.creationTime = System.currentTimeMillis();
    }
    
    /**
     * Prepares the TextElement for rendering.
     */
    public void activate() {
        this.active = true;
        long testLong = this.creationTime;
        if (this.duration > 0) {
            Timer levelTitleGTFO = new Timer();
            levelTitleGTFO.schedule(new TimerTask() {
                public void run() {
                    for (byte i = 0; i < gameState.currentTextElements.length; i++) {
                        if (gameState.currentTextElements[i] != null) {
                            if (gameState.currentTextElements[i].creationTime == testLong) {
                                gameState.currentTextElements[i].cull();
                            }
                        }
                    }
                }
            }, this.duration);
        }
    }
    
    /**
     * Prepares the TextElement for culling.
     */
    public void cull() {
        this.active = false;
        this.x = 32767;
        this.y = 32767;
    }
    
    /**
     * Changes the text that the TextElement will display.
     * @param text The text that the TextElement will display.
     */
    public void setText(String text) {
        this.text = text;
    }
    
    /**
     * Changes the text that the TextElement will display.
     * @param x The x coordinate of the text's origin point.
     * @param y The y coordinate of the text's origin point.
     */
    public void moveTo(int x, int y) {
        this.x = (short) x;
        this.y = (short) y;
    }
    
    /**
     * Changes the color of the text.
     * @param textColor The color code of the text.
     */
    public void setColor(int textColor) {
        this.textColor = (byte) textColor;
    }
    
}
